// $Id: MsgBean.java,v 1.9 2004/03/23 00:00:05 gca Exp $
package com.spamnbeans.msg;

import javax.ejb.*;

import java.util.logging.*;


/**
 * The Entity bean for maintaining a message (Msg)
 * <p>This represents a persistent message which has:
 * <ul>
 * <li>MsgID 		- unique identifer for the message
 * <li>ClientID 	- indicates which client submitted the message
 * <li>ServerID 	- indicates which server processed the message
 * <li>TimeStamp 	- indicates when the client submitted the message
 * <li>MsgSize		- indicates the size in bytes of the message
 * </ul>
 * 
 * @author Gary Ackley, <a href="mailto:gca@andrew.cmu.edu">gca@andrew.cmu.edu
 *         </a>
 * @version $Revision: 1.9 $
 */
public abstract class MsgBean implements javax.ejb.EntityBean {

	protected EntityContext ctx;
	private static Logger logger =
		Logger.getLogger(MsgBean.class.getName());

	public MsgBean() {		
	}
	
	// -------------------------------------------------------------------
	// EJB Required Methods
	// - These are only called by container, and never by clients
	// -------------------------------------------------------------------
	
	/**
	 * This is the initialization method that corresponds to the 
	 * create() method in the Home Interface.
	 * - When the client calls the Home Object's create() method, 
	 *   the Home Object then calls this ejbCreate() method.
	 * - This is where the bean's fields are initialized with parameters
	 *   passed from the client, so that the Container can create the
	 *   corresponding database entries in the subclass after this
	 *   method completes.
	 */
	public Integer ejbCreateMsg(
			Integer messageID, Integer clientID, Integer serverID,
			String timeStamp, Integer msgSize, Integer msgState) 
	throws CreateException {
	    //System.out.println("MsgBean ejbCreateMsg()");
		
		setMsgID(messageID);
		setClientID(clientID);
		setServerID(serverID);
		setTimeStamp(timeStamp);
		setMsgSize(msgSize);
		setMsgState(msgState);
		
		return getMsgID();
	}
	
	/**
	 * Called after ejbPostCreateMsg()
	 * - Now the bean can retreive its EJBObject from its context
	 *   and pass it a 'this' argument.
	 */
	public void ejbPostCreateMsg(
			Integer messageID, Integer clientID, Integer serverID,
			String timeStamp, Integer msgSize, Integer msgState) {
	    logger.info("MsgBean ejbPostCreateMsg()");
	}
	
	/**
	 * Called by container.
	 * - Implementation can acquire needed resources.
	 */
	public void ejbActivate() {
	    logger.info("MsgBean ejbActivate()");
	}
	
	/**
	 * Called by container.
	 * - Release resources held for passivation.
	 */
	public void ejbPassivate() {
	    logger.info("MsgBean ejbPassivate()");
	}
	
	/**
	 * Called from the container.
	 * - Updates the entity bean instance reflect the current 
	 *   value of stored in the database.
	 * - Since this is using Container-Managed Persistence, we
	 *   can leave this method blank.  The EJB container will 
	 *   automatically load the data in the subclass.  
	 */
	public void ejbLoad() {
	    logger.info("MsgBean ejbLoad()");
	}
	
	/**
	 * Called from the container.
	 * - Updates the database to reflect the current values of
	 *   this in-memory instance representation.
	 * - Since this is using Container-Managed Persistence, we
	 *   can leave this method blank.  The EJB container will
	 *   automatically save the data in the subclass. 
	 */
	public void ejbStore() {
	    logger.info("MsgBean ejbStore()");
		// Using CMP so this can stay empty
		// - Container will auto-save in the subclass
	}
	
	/**
	 * EJB container calls this method just before it
	 * removes the Entity bean from the database.
	 * - Corresponds to when client calls home.remove() 
	 */
	public void ejbRemove() {
	    logger.info("MsgBean ejbRemove()");
	}
	
	/**
	 * Called by the container.
	 * - Associates this bean instance with a context.
	 * - Once done, the instance can query the context
	 *   for environment info. 
	 */
	public void setEntityContext(EntityContext ctx) {
	    logger.info("MsgBean setEntityContext()");
		this.ctx = ctx;
	}
	
	/**
	 * Called by the container.
	 * - Disassociates this bean instance with a 
	 *   particular context environment.
	 */
	public void unsetEntityContext() {
	    logger.info("MsgBean unsetEntityContext()");
		this.ctx = null;
	}

	// -------------------------------------------------------------------
	// Abstract get/set methods
	// -------------------------------------------------------------------
	
	public abstract Integer getMsgID();
	public abstract void setMsgID(Integer msgID);
	
	public abstract Integer getClientID();
	public abstract void setClientID(Integer clientID);

	public abstract Integer getServerID();
	public abstract void setServerID(Integer serverID);

	public abstract String getTimeStamp();
	public abstract void setTimeStamp(String timeStamp);
	
	public abstract Integer getMsgSize();
	public abstract void setMsgSize(Integer msgSize);
	
	public abstract Integer getMsgState();
	public abstract void setMsgState(Integer msgState);
	
	// No Finder methods
	// - these are implemented by the container	

}
