// $RCSfile: SpamCBean.java,v $
// $Author: cfry $ 
// $Date: 2004/03/03 03:25:09 $
// $Revision: 1.1 $

package com.spamnbeans.spam;

//import com.spamnbeans.client.*;
//import com.spamnbeans.server.*;

//import javax.rmi.PortableRemoteObject;
import java.rmi.RemoteException;
//import javax.naming.InitialContext;
//import javax.naming.Context;
//import javax.ejb.EJBException;
//import javax.ejb.FinderException;
//import javax.ejb.CreateException;
//import java.lang.Exception;

import java.io.*;
import java.net.*;

/**
 * Define the SpamC SessionBean methods.
 * 
 * <p>The SpamC Bean monitors spamd and feeds it messages over TCP/IP.
 *
 * @author Charles Fry,
 *         <a href="mailto:cfry@ece.cmu.edu">cfry@ece.cmu.edu</a>
 * @version 1.0
 */
public class SpamCBean implements javax.ejb.SessionBean 
{
    // the hex values of the capital letters in Spam-n-Beans
    private static final int port = 5342;
    private static final int SOCKET_TIMEOUT = 3 * 60 * 1000; // 3 minutes
    private static final String cmd = "/afs/ece/usr/cfry/classes/" +
	"middleware/spamassassin/usr/bin/spamd --port "+port;
	private static final int RETRY_COUNT = 2;
	private static final int SPAWN_SLEEP = 5 * 1000;

    private Process spamd;

    /**
     * Not implemented
     */
    public void ejbCreate() throws RemoteException { }

    /**
     * Retire the SpamC SessionBean and shut down spamd
     */
    public void ejbRemove() {
        if (spamd != null) {
            System.out.println("stopping spamd");
            spamd.destroy();
        }
    }

    /**
     * Not implemented
     */
    public void ejbActivate(){}
    
    /**
     * Not implemented
     */
    public void ejbPassivate(){} // TODO could stop spamd; but why?
    
    /**
     * Not implemented
     */
    public void setSessionContext(javax.ejb.SessionContext cntx){}

    /**
     * Start spamd if necessary, and get a socket connection to it.
     * @return A Socket connected to spamd
     */
    private synchronized Socket getSpamdSocket() {
        Socket socket = null;
        System.out.println("connecting to spamd");
		try {
			socket = new Socket("localhost", port);
			socket.setSoTimeout(SOCKET_TIMEOUT);
		}
		catch (ConnectException e) {
			// couldn't connect, assume spamd isn't running
			try {
				if (spamd != null) {
					System.out.println("stopping spamd");
					spamd.destroy();
					spamd = null; // don't try to stop again
				}
				System.out.println("spawning spamd");
				spamd = Runtime.getRuntime().exec(cmd);
				try {
					// give spamd a chance to spawn
					Thread.sleep(SPAWN_SLEEP);
				}
				catch (InterruptedException ie) { } //ignore
			}
			catch (Throwable t) {
				System.err.println("Exception restarting spamd: "+t);
				/*
				if (MPClient.STACK_TRACES) {
					t.printStackTrace();
				}
				*/
			}
		}
		catch (Throwable t) {
			System.err.println("Exception connecting to spamd: "+t);
			/*
			if (MPClient.STACK_TRACES) {
				t.printStackTrace();
			}
			*/
		}

		return socket;
    }

	private void sendMessage(String message, OutputStream out) throws IOException {
		System.out.println("sending message");
		byte[] msgBytes = message.getBytes();
		String header = "PROCESS SPAMC/1.3\r\nContent-length: "
			+ msgBytes.length+"\r\n\r\n";
		out.write(header.getBytes());
		out.write(msgBytes);
		out.flush();
	}

	private String readHeaders(InputStream in) throws IOException {
		// Format is "Spam: z; y / x" ex "Spam: true; 6.7 / 5.0"
		System.out.println("reading response");
		BufferedReader bin = new BufferedReader(new 
		  InputStreamReader(in));
		String inputLine = bin.readLine();
		if (!inputLine.equals("SPAMD/1.1 0 EX_OK")) {
			throw new IOException("invalid header: "+inputLine);
		}
		inputLine = bin.readLine(); // Content-length: 260
		if (inputLine == null) {
			throw new IOException("Unexpected EOF from spamd");
		}
		inputLine = bin.readLine(); // Spam: False ; 2.9 / 5.0
		if (inputLine == null) {
			throw new IOException("Unexpected EOF from spamd");
		}
		inputLine = bin.readLine(); // [blank line]
		if (inputLine == null) {
			throw new IOException("Unexpected EOF from spamd");
		}
		/* Java is dumb; I want to readlines to get size, 
		   and then read bytes
		   String[] split = inputLine.split(" ");
		   int length = Integer.parseInt(split[1]);
		   byte[] bytes = new byte[length];
		   in.read(bytes);
		   return new String(bytes);
		*/
		StringBuffer buf = new StringBuffer();
		while ((inputLine = bin.readLine()) != null) {
			//System.out.println("read: "+inputLine);
			if (inputLine.startsWith("X-Spam-")) {
				buf.append(inputLine);
				buf.append("\n");
			}
		}
		return buf.toString();
	}

    /**
     * Process a single e-mail message
     * @param message The full text of the e-mail to check
     * @return The extra e-mail headers generated
     */
	public String processMessage(String message) throws SpamException {
		for (int i = 0; i < RETRY_COUNT; i++) {
			Socket socket = getSpamdSocket();
			if (socket == null) {
				System.out.println("Unable to connect to spamd");
				continue;
			}
			try {
				System.out.println("opening streams");
				BufferedInputStream in = new 
				  BufferedInputStream(socket.getInputStream());
				BufferedOutputStream out = new 
				  BufferedOutputStream(socket.getOutputStream());

				sendMessage(message, out);
				return readHeaders(in);
			}
			catch (IOException e) {
				System.out.println("Error communicating with spamd: "+e);
			}
			catch (Throwable t) {
				System.out.println("Exception while communicating with spamd: "+t);
			}
			finally {
				try {
					socket.close();
				} catch (IOException e) {
					System.out.println("Error while closing socket: "+e); }
			}
		}

		// would have returned by now if succesfull
		throw new SpamException("Unable to succesfully communicate with spamd");
	}
}
